/*
 * Gesture Manager
 * Implementation file.
 * By Daniel Meja V.
 */


#include "gestures.h"
#include "string.h"
#include <stdio.h> // DEBUGGING, REMOVE!
#include "uart.h" // DEBUGGING, REMOVE!



// UART file descriptor FOR DEBUGGIN PURPOSES! REMOVE!
// putchar and getchar are in uart.c
extern FILE uart_str;

// PROTOTYPES
byte normalize_xpos_value(unsigned int value);
byte normalize_ypos_value(unsigned int value);
void remove_trailing_zeros(gesture_data_t *src, gesture_data_t *dst, int size, int *new_size);
int valid_touch(gesture_data_t data);
int get_abs(int);
void print_gesture(gesture_data_t *gest, int size); // FOR DEBUGGING


// PUBLIC FUNCTIONS

/*
 * Checks if the two given gestures match
 */
int gesture_check_match(gesture_data_t *g1, unsigned int g1_size,
					gesture_data_t *g2, unsigned int g2_size) {
	int i, diffx = 0, diffy = 0;
	int g1_newsize, g2_newsize, size_diff;
	unsigned int min_size;
	gesture_data_t g1_new[g1_size]; // Processed version of the data
	gesture_data_t g2_new[g2_size];

	remove_trailing_zeros(g1, g1_new, g1_size, &g1_newsize);
	remove_trailing_zeros(g2, g2_new, g2_size, &g2_newsize);

	size_diff = get_abs(g1_newsize-g2_newsize);
	//if (size_diff > SIZE_THRESHOLD)
		//return 0;

	min_size = g1_newsize < g2_newsize ? g1_newsize : g2_newsize;
	fprintf(stdout, "GESTURE 1 NEW:\n\n");
	print_gesture(g1_new, g1_newsize);
	fprintf(stdout, "GESTURE 2 NEW:\n\n");
	print_gesture(g2_new, g2_newsize);
	for (i = 0; i < min_size; i++) {
		diffx += get_abs((*(g1_new+i)).xpos-(*(g2_new+i)).xpos);
		diffy += get_abs((*(g1_new+i)).ypos-(*(g2_new+i)).ypos);
	}
	fprintf(stdout, "The x diff was %d, and the y diff was %d\n\n", diffx, diffy);
	return diffx < 50 && diffy < 50; // TODO Normalize with respect to gesture size
}

/*
 * Transforms the touchpad packet into the data
 * representation that is used for gesture recognition.
 */
gesture_data_t touchpad2gesture_data(tp_abs_packet_t packet) {
	gesture_data_t gdata;
	gdata.xpos = normalize_xpos_value(packet.xpos);
	gdata.ypos = normalize_ypos_value(packet.ypos);
	gdata.zpress = packet.zpress;
	return gdata;
}




// PRIVATE FUNCTIONS

/*
 * Normalizes the touchpad x position value
 * into a discrete representation used
 * for gestures recognition
 */
byte normalize_xpos_value(unsigned int value) {
	byte xpos;
	int step = (TP_HIGHER_BEZEL_XPOS - TP_LOWER_BEZEL_XPOS)/ GRID_SIZE_X;
	value = value < TP_LOWER_BEZEL_XPOS ? 0 : value - TP_LOWER_BEZEL_XPOS;
	xpos = value/step;
	return xpos < GRID_SIZE_X ? xpos : GRID_SIZE_X -1;
}

/*
 * Normalizes the touchpad y position value
 * into a discrete representation used
 * for gestures recognition
 */
byte normalize_ypos_value(unsigned int value) {
	byte ypos;
	int step = (TP_HIGHER_BEZEL_YPOS - TP_LOWER_BEZEL_YPOS)/ GRID_SIZE_Y;
	value = value < TP_LOWER_BEZEL_YPOS ? 0 : value - TP_LOWER_BEZEL_YPOS;
	ypos = value/step;
	return ypos < GRID_SIZE_Y ? ypos : GRID_SIZE_Y -1;
}

/*
 * Removes the initial and final zeroes that come from the
 * time the user takes from/to pressing the bottom
 */
void remove_trailing_zeros(gesture_data_t *src, gesture_data_t *dst, int size, int *new_size) {
	int i, initialz_done = 0, num_initialz = 0, num_finalz = 0;
	*new_size = 0;
	// Remove initial zeros
	for (i = 0; i < size; i++) {
		if (valid_touch(*(src+i)) && initialz_done == 0) {
			initialz_done = 1;
			num_initialz = i;
		}
		if (initialz_done == 1) {
			memcpy(dst+i-num_initialz, src+i, sizeof(gesture_data_t));
		}
	}
	// Count the number of final zeros
	for (i = size-num_initialz-1; valid_touch(*(dst+i)) == 0; i--) {
		num_finalz++;
	}
	*new_size = size - num_initialz - num_finalz;
}

/*
 * Returns the absolute value of x
 */
int get_abs(int x) {
	return x >= 0 ? x : -x;
}

/*
 * Returns 1, if the data represents a valid touch.
 * i.e zpress higher than the threshold
 */
int valid_touch(gesture_data_t data) {
	return data.zpress >= ZPRESS_TOUCH_THRESHOLD;
}

/*
 * Prints the whole gesture
 * USED FOR DEBUGGING, REMOVE
 */
void print_gesture(gesture_data_t *gest, int size) {
	int i;
	fprintf(stdout, "\n\nThe gesture is:\n\n\n");
	for (i = 0; i < size; i++) {
		fprintf(stdout, "GESTURE UNIT %d:\nx:%d\ny:%d\nz:%d\n\n",
				i, gest[i].xpos,gest[i].ypos, gest[i].zpress);
	}
	fprintf(stdout, "\n\n\n");
}


