/*
 * Gesture Manager
 * Interface file.
 * By Daniel Meja V.
 */

#include "touchpad.h"

/*
 * Gestures recognition.
 * Interface file.
 */

#ifndef __GESTURES_H__
#define __GESTURES_H__

// Grid sizes for touchpad discrete position values
#define GRID_SIZE_X 32
#define GRID_SIZE_Y 24

#define ZPRESS_TOUCH_THRESHOLD 10 // Under which the data is not considered
#define SIZE_THRESHOLD 20 // Size difference after which 2 gestures do not match
						  // Must depend on the sampling frequency
						  // For 50ms freq, 20 -> 1 sec diff

/*
 * Representation of the touchpad data
 * from the gestures point of view.
 * x and y are normalized to values in a grid,
 * and w is not used.
 */
typedef struct {
	byte xpos;
	byte ypos;
	byte zpress;
} gesture_data_t;

/*
 * Checks if the two given gestures match
 */
int gesture_check_match(gesture_data_t *g1, unsigned int g1_size,
						gesture_data_t *g2, unsigned int g2_size);
/*
 * Normalizes the touchpad packet into the data
 * representation that is used for gesture recognition.
 */
gesture_data_t touchpad2gesture_data(tp_abs_packet_t packet);







#endif
