/*
 * PS/2 Controller
 * Interface file.
 * By Daniel Meja V.
 */


#include <avr/io.h>

#ifndef __PS2_H__

#define __PS2_H__


/*
 * NOTE: Set the clock and data macros
 * with the appropriate pins of
 * your system
 */
#define PS2DATA_PORT_READ PIND
#define PS2DATA_PORT_WRITE PORTD
#define PS2DATA_PORTDIR DDRD
#define PS2DATA_PIN PIND3
#define PS2CLOCK_PORT_READ PIND
#define PS2CLOCK_PORT_WRITE PORTD
#define PS2CLOCK_PORTDIR DDRD
#define PS2CLOCK_PIN PIND2

#define PS2ERROR_PORT PORTB
#define PS2ERR0R_PIN PINB7


// Command macros
#define PS2COMMAND_RESET 0xFF
#define PS2COMMAND_RESEND 0xFE // Device response when receiving a wrong command
#define PS2COMMAND_ERROR 0xFC // Device response when receiving a wrong command for the 2nd time
#define PS2COMMAND_SET_DEFAULTS 0xF6
#define PS2COMMAND_DISABLE 0xF5
#define PS2COMMAND_ENABLE 0xF4
#define PS2COMMAND_SET_SAMPLE_RATE 0xF3
#define PS2COMMAND_READ_DEVICE_TYPE 0xF2
#define PS2COMMAND_SET_REMOTE_TYPE 0xF0
#define PS2COMMAND_SET_WRAP_MODE 0xEE
#define PS2COMMAND_RESET_WRAP_MODE 0xEC
#define PS2COMMAND_READ_DATA 0xEB
#define PS2COMMAND_SET_STREAM_MODE 0xEA
#define PS2COMMAND_STATUS_REQUEST 0xE9
#define PS2COMMAND_SET_RESOLUTION 0xE8
#define PS2COMMAND_SET_SCALING_21 0xE7
#define PS2COMMAND_SET_SCALING_11 0xE6
#define PS2COMMAND_ACK 0xFA


typedef unsigned char byte;


/*
 * Returns a byte of data received from
 * a PS/2 device.
 * Waits for the clock to go low.
 */
byte ps2_get();

/*
 * Returns a byte of data received from
 * a PS/2 device, without waiting for the clock
 * pin to go low. This should happen before calling
 * the function.
 */
byte ps2_receive();

/*
 * Sends a byte of data to
 * a PS/2 device
 */
void ps2_send(byte value);

/*
 * Disable PS/2 device by
 * pulling the clock down
 */
void ps2_inhibit_device();

/*
 * Enables PS/2 device
 */
void ps2_disinhibit_device();

/*
 * Initializes the ports used to
 * access the P2/2 device
 */
void ps2_init_ports();

#endif
