/*
 * Simplified implementation of a dynamic list of int8_ts.
 * Implementation file.
 * By Daniel Meja V.
 */

#include "list.h"
#include <stdlib.h>
#include <stdint.h>
/*
 * Creates and returns a value with the
 * specified initial_size
 */
list_t list_create(unsigned int initial_max_size) {
	list_t list = (void *)malloc(sizeof(struct list_st));
	list->size = 0;
	list->max_size = initial_max_size;
	list->values = (void *)malloc(initial_max_size*sizeof(int8_t));
	return list;
}

/*
 * Adds value to the end of the list
 */
void list_add(list_t list, int8_t value) {
	if (list->size == list->max_size) {
		list->max_size <<= 1;
		list->values = (void *)realloc(list->values, list->max_size*sizeof(int8_t));
	}
	*(list->values + list->size) = value;
	list->size++;
}

/*
 * Sets the element at pos with the specified value
 * Returns -1 if error
 */
int list_set(list_t list, unsigned int pos, int8_t value) {
	if (pos >= list->size)
		return -1;
	*(list->values + pos) = value;
	return 0;

}

/*
 * Gets the element at pos.
 * pos must be less than the list size,
 * otherwise an invalid -1 is returned
 */
int8_t list_get(list_t list, unsigned int pos) {
	return pos < list->size ? *(list->values + pos) : -1;
}

/*
 * Returns the size of the list
 */
unsigned int list_size(list_t list) {
	return list->size;
}

/*
 * Frees the list
 */
void list_free(list_t list) {
	free(list->values);
	free(list);

}
